import inspect

def enhance_ctrl_class(cls, path, args, kwargs):
    cls.METAROUTE_CONTROLLER_PATH   = path
    cls.METAROUTE_CONTROLLER_ARGS   = args
    cls.METAROUTE_CONTROLLER_KWARGS = kwargs
    return cls

def Controller(*args, **kwargs):
    if not inspect.isclass(args[0]):
        def d(cls):
            return enhance_ctrl_class(cls, args[0], args[1:], kwargs)
        return d
    else:
        return enhance_ctrl_class(args[0], "", [], {})

def Route(path = "", *args, **kwargs):
    def d(f):
        f.METAROUTE_ACTION_PATHS = getattr(f, "METAROUTE_ACTION_PATHS", []) + [(path, args, kwargs)]
        return f
    return d

def Error(ex = ""):
    def d(f):
        f.METAROUTE_ERROR_EXCEPTIONS = getattr(f, "METAROUTE_ERROR_EXCEPTIONS", []) + [ex]
        return f
    return d




try:
    from flask import _app_ctx_stack as stack
except ImportError:
    from flask import _request_ctx_stack as stack #@UnusedImport

from flask_metaroute.func import attach_controllers
import importlib
    
class MetaRoute(object):
    
    def __init__(self, app = None, ctrl_pkg = None):
        if app is not None:
            self.app = app
            self.init_app(self.app, ctrl_pkg)
        else:
            self.app = None

    def init_app(self, app, ctrl_pkg = None):
        pkg = ctrl_pkg or app.config['METAROUTE_CONTROLLERS_PKG']
        if isinstance(pkg, str):
            pkg = importlib.import_module(pkg)
            
        
        attach_controllers(app, pkg)



import types as t
import os, imp

def scan_folder_recurse(folder, excl_names=['__']):
    """ Recurse search for PY files in given folder """                                                                                                                                   
    all_files = []                                                                                                                                                                      
    for root, d, files in os.walk(folder):                                              #@UnusedVariable
        filelist = [os.path.join(root, fi) for fi in files if fi.endswith('.py')
                    and not any(fi.startswith(prefix) for prefix in excl_names)]
        all_files += filelist
    return all_files

def scan_package(p):
    for x in dir(p):
        if not x.startswith("_"):
            o = getattr(p, x)        
            if   type(o) == t.ClassType:
                print "Yielding1 [%s.%s]" % (o.__module__, o.__name__)
                yield o
#            elif type(o) == t.ModuleType:
#                for xo in scan_package(o):
#                    print "Yielding2 [%s.%s]" % (o.__module__, o.__name__)
#                    yield xo

def attach_controllers(app, pkg):
    pkg_path  = pkg.__path__[0]
    pkg_name  = pkg.__name__
    
    all_files = scan_folder_recurse(pkg_path)
    for f in all_files:
        pkg = f[len(pkg_path):]
        pkg = pkg.strip("/")[:-3]
        
        if len(pkg):
            pkg = pkg_name + "." + pkg
        
        attach_controller(app, imp.load_source(pkg, f))
        
        
def attach_controller(app, mdl):
    for cls in scan_package(mdl):
        if cls.__module__.startswith(mdl.__name__):
            cpath = getattr(cls, "METAROUTE_CONTROLLER_PATH", None)
            if cpath is not None:
                ctrl    = cls()
                methods = dir(ctrl)
                for meth in methods:
                    m  = getattr(cls, meth, None)
                    
                    for path, args, opts in getattr(m, "METAROUTE_ACTION_PATHS", []):
                        xf = cls.__dict__[m.__name__].__get__(ctrl, cls)
                        opts['endpoint'] = cls.__module__ + "." + cls.__name__ + "." + xf.__name__
                        app.route(cpath + path, **opts)(cls.__dict__[m.__name__].__get__(ctrl, cls))
                        print "Enhanced [%s]" % meth
    
                    for ex in getattr(m, "METAROUTE_ERROR_EXCEPTIONS", []):
                        app._register_error_handler(None, ex, cls.__dict__[m.__name__].__get__(ctrl, cls))
            






"""
Flask-MetaRoute
-------------

Flask-MetaRoute adds some useful decorators for routing
"""
from setuptools import setup

setup(
    name='Flask-MetaRoute',
    version='1.0',
    url='http://code.google.com/p/flask-metaroute/',
    license='BSD',
    author='Orca',
    author_email='deep.orca@gmail.com',
    description='Extra routing capabilities for Flask',
    long_description=__doc__,
    packages=['flask_metaroute'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)